@@ -160,7 +160,7 @@ bool is_word_mbchar(const char *c, bool allow_punct)
 
 	if (word_chars != NULL && *word_chars != '\0') {
 		char symbol[MAXCHARLEN + 1];
-		int symlen = parse_mbchar(c, symbol, NULL);
+		int symlen = collect_char(c, symbol);
 
 		symbol[symlen] = '\0';
 		return (strstr(word_chars, symbol) != NULL);
@@ -200,11 +200,12 @@ char control_mbrep(const char *c, bool isdata)
 		return control_rep(*c);
 }
 
-/* This function is equivalent to wcwidth() for multibyte characters. */
+#ifdef ENABLE_UTF8
+/* Return the width in columns of the given (multibyte) character. */
 int mbwidth(const char *c)
 {
-#ifdef ENABLE_UTF8
-	if (use_utf8) {
+	/* Ask for the width only when the character isn't plain ASCII. */
+	if ((signed char)*c <= 0) {
 		wchar_t wc;
 		int width;
 
@@ -213,41 +214,41 @@ int mbwidth(const char *c)
 
 		width = wcwidth(wc);
 
-		if (width == -1)
+		if (width < 0)
 			return 1;
 
 		return width;
 	} else
-#endif
 		return 1;
 }
+#endif
 
-/* Convert the Unicode value in chr to a multibyte character, if possible.
+/* Convert the Unicode value in code to a multibyte character, if possible.
  * If the conversion succeeds, return the (dynamically allocated) multibyte
  * character and its length.  Otherwise, return an undefined (dynamically
  * allocated) multibyte character and a length of zero. */
-char *make_mbchar(long chr, int *chr_mb_len)
+char *make_mbchar(long code, int *length)
 {
-	char *chr_mb;
+	char *mb_char;
 
 #ifdef ENABLE_UTF8
 	if (use_utf8) {
-		chr_mb = charalloc(MAXCHARLEN);
-		*chr_mb_len = wctomb(chr_mb, (wchar_t)chr);
+		mb_char = charalloc(MAXCHARLEN);
+		*length = wctomb(mb_char, (wchar_t)code);
 
 		/* Reject invalid Unicode characters. */
-		if (*chr_mb_len < 0 || !is_valid_unicode((wchar_t)chr)) {
+		if (*length < 0 || !is_valid_unicode((wchar_t)code)) {
 			IGNORE_CALL_RESULT(wctomb(NULL, 0));
-			*chr_mb_len = 0;
+			*length = 0;
 		}
 	} else
 #endif
 	{
-		*chr_mb_len = 1;
-		chr_mb = mallocstrncpy(NULL, (char *)&chr, 1);
+		mb_char = measured_copy((char *)&code, 1);
+		*length = 1;
 	}
 
-	return chr_mb;
+	return mb_char;
 }
 
 /* Return the length (in bytes) of the character located at *pointer. */
@@ -264,50 +265,88 @@ int char_length(const char *pointer)
 		return 1;
 }
 
-/* Parse a multibyte character from buf.  Return the number of bytes
- * used.  If chr isn't NULL, store the multibyte character in it.  If
- * col isn't NULL, add the character's width (in columns) to it. */
-int parse_mbchar(const char *buf, char *chr, size_t *col)
+/* Return the number of (multibyte) characters in the given string. */
+size_t mbstrlen(const char *pointer)
 {
-	int length;
+	size_t count = 0;
+
+	while (*pointer != '\0') {
+#ifdef ENABLE_UTF8
+		if ((signed char)*pointer < 0) {
+			int length = mblen(pointer, MAXCHARLEN);
+
+			pointer += (length < 0 ? 1 : length);
+		} else
+#endif
+			pointer++;
+
+		count++;
+	}
+
+	return count;
+}
+
+/* Return the length (in bytes) of the character at the start of the
+ * given string, and return a copy of this character in *thechar. */
+int collect_char(const char *string, char *thechar)
+{
+	int charlen;
 
 #ifdef ENABLE_UTF8
 	/* If this is a UTF-8 starter byte, get the number of bytes of the character. */
-	if ((signed char)*buf < 0) {
-		length = mblen(buf, MAXCHARLEN);
+	if ((signed char)*string < 0) {
+		charlen = mblen(string, MAXCHARLEN);
 
 		/* When the multibyte sequence is invalid, only take the first byte. */
-		if (length <= 0)
-			length = 1;
+		if (charlen <= 0)
+			charlen = 1;
 	} else
 #endif
-		length = 1;
+		charlen = 1;
 
-	/* When requested, store the multibyte character in chr. */
-	if (chr != NULL)
-		for (int i = 0; i < length; i++)
-			chr[i] = buf[i];
+	for (int i = 0; i < charlen; i++)
+		thechar[i] = string[i];
 
-	/* When requested, add the width of the character to col. */
-	if (col != NULL) {
-		/* If we have a tab, compute its width in columns based on the
-		 * current value of col. */
-		if (*buf == '\t')
-			*col += tabsize - *col % tabsize;
-		/* If we have a control character, it's two columns wide: one
-		 * column for the "^", and one for the visible character. */
-		else if (is_cntrl_mbchar(buf))
-			*col += 2;
-		/* If we have a normal character, get its width normally. */
-		else if (length == 1)
-			*col += 1;
+	return charlen;
+}
+
+/* Return the length (in bytes) of the character at the start of
+ * the given string, and add this character's width to *column. */
+int advance_over(const char *string, size_t *column)
+{
 #ifdef ENABLE_UTF8
-		else
-			*col += mbwidth(buf);
-#endif
-	}
+	if ((signed char)*string < 0) {
+		int charlen = mblen(string, MAXCHARLEN);
 
-	return length;
+		if (charlen > 0) {
+			if (is_cntrl_mbchar(string))
+				*column += 2;
+			else
+				*column += mbwidth(string);
+		} else {
+			charlen = 1;
+			*column += 1;
+		}
+
+		return charlen;
+	}
+#endif
+
+	if ((unsigned char)*string < 0x20) {
+		if (*string == '\t')
+			*column += tabsize - *column % tabsize;
+		else
+			*column += 2;
+	} else if (*string == 0x7F)
+		*column += 2;
+#ifndef ENABLE_UTF8
+	else if (0x7F < (unsigned char)*string && (unsigned char)*string < 0xA0)
+		*column += 2;
+#endif
+	else
+		*column += 1;
+
+	return 1;
 }
 
 /* Return the index in buf of the beginning of the multibyte character
@@ -346,7 +385,7 @@ size_t step_left(const char *buf, size_t pos)
 		return before - charlen;
 	} else
 #endif
-	return (pos == 0 ? 0 : pos - 1);
+		return (pos == 0 ? 0 : pos - 1);
 }
 
 /* Return the index in buf of the beginning of the multibyte character
@@ -487,27 +526,6 @@ char *mbrevstrcasestr(const char *haystack, const char *needle,
 		return revstrcasestr(haystack, needle, pointer);
 }
 
-/* Count the number of (multibyte) characters in the given string. */
-size_t mbstrlen(const char *pointer)
-{
-	size_t count = 0;
-
-	while (*pointer != '\0') {
-#ifdef ENABLE_UTF8
-		if ((signed char)*pointer < 0) {
-			int length = mblen(pointer, MAXCHARLEN);
-
-			pointer += (length < 0 ? 1 : length);
-		} else
-#endif
-			pointer++;
-
-		count++;
-	}
-
-	return count;
-}
-
 #if !defined(NANO_TINY) || defined(ENABLE_JUSTIFY)
 /* This function is equivalent to strchr() for multibyte strings. */
 char *mbstrchr(const char *string, const char *chr)
@@ -592,7 +610,7 @@ bool has_blank_char(const char *string)
 	char symbol[MAXCHARLEN];
 
 	while (*string != '\0') {
-		string += parse_mbchar(string, symbol, NULL);
+		string += collect_char(string, symbol);
 
 		if (is_blank_mbchar(symbol))
 			return TRUE;
@@ -602,6 +620,15 @@ bool has_blank_char(const char *string)
 }
 #endif /* ENABLE_NANORC && (!NANO_TINY || ENABLE_JUSTIFY) */
 
+/* Return TRUE when the given string is empty or consists of only blanks. */
+bool white_string(const char *string)
+{
+	while (*string != '\0' && (is_blank_mbchar(string) || *string == '\r'))
+		string += char_length(string);
+
+	return !*string;
+}
+
 #ifdef ENABLE_UTF8
 /* Return TRUE if wc is valid Unicode, and FALSE otherwise. */
 bool is_valid_unicode(wchar_t wc)
