@@ -22,11 +22,11 @@
 
 #include "proto.h"
 
+#ifdef ENABLE_HELP
+
 #include <errno.h>
 #include <string.h>
 
-#ifdef ENABLE_HELP
-
 static char *help_text = NULL;
 		/* The text displayed in the help window. */
 
@@ -64,7 +64,7 @@ void wrap_help_text_into_buffer(void)
 		do {
 			openfile->current->next = make_new_node(openfile->current);
 			openfile->current = openfile->current->next;
-			openfile->current->data = mallocstrcpy(NULL, "");
+			openfile->current->data = copy_of("");
 		} while (*(++ptr) == '\n');
 	}
 
@@ -88,8 +88,8 @@ void wrap_help_text_into_buffer(void)
 	openfile->edittop = openfile->current;
 }
 
-/* Our main help-viewer function. */
-void do_help(void)
+/* Assemble a help text, display it, and allow scrolling through it. */
+void show_help(void)
 {
 	int kbinput = ERR;
 	functionptrtype func;
@@ -103,7 +103,7 @@ void do_help(void)
 #ifdef ENABLE_COLOR
 	char *was_syntax = syntaxstr;
 #endif
-	char *saved_answer = (answer != NULL) ? strdup(answer) : NULL;
+	char *saved_answer = (answer != NULL) ? copy_of(answer) : NULL;
 		/* The current answer when the user invokes help at the prompt. */
 	unsigned stash[sizeof(flags) / sizeof(flags[0])];
 		/* A storage place for the current flag settings. */
@@ -140,8 +140,9 @@ void do_help(void)
 #endif
 	curs_set(0);
 
-	/* Compose the help text from all the pieces. */
+	/* Compose the help text from all the relevant pieces. */
 	help_init();
+
 	inhelp = TRUE;
 	location = 0;
 	didfind = 0;
@@ -150,8 +151,7 @@ void do_help(void)
 
 	/* Extract the title from the head of the help text. */
 	length = break_line(help_text, MAX_BUF_SIZE, TRUE);
-	title = charalloc(length * sizeof(char) + 1);
-	strncpy(title, help_text, length);
+	title = measured_copy(help_text, length + 1);
 	title[length] = '\0';
 
 	titlebar(title);
@@ -247,7 +247,6 @@ void do_help(void)
 	inhelp = FALSE;
 
 	curs_set(0);
-	refresh_needed = TRUE;
 
 	if (ISSET(NO_HELP)) {
 		currmenu = oldmenu;
@@ -262,7 +261,10 @@ void do_help(void)
 		browser_refresh();
 	else
 #endif
+	{
 		titlebar(NULL);
+		edit_refresh();
+	}
 }
 
 /* Allocate space for the help text for the current menu, and concatenate
@@ -547,7 +549,6 @@ functionptrtype parse_help_input(int *kbinput)
 {
 	if (!meta_key) {
 		switch (*kbinput) {
-			case BS_CODE:
 			case '-':
 				return do_page_up;
 			case ' ':
@@ -600,14 +601,13 @@ size_t help_line_len(const char *ptr)
 	else
 		return 1;
 }
-
 #endif /* ENABLE_HELP */
 
-/* Start the help viewer. */
-void do_help_void(void)
+/* Start the help viewer, or indicate that there is no help. */
+void do_help(void)
 {
 #ifdef ENABLE_HELP
-	do_help();
+	show_help();
 #else
 	if (currmenu == MMAIN)
 		say_there_is_no_help();
