@@ -53,8 +53,8 @@ static int oldinterval = -1;
 		/* Used to store the user's original mouse click interval. */
 #endif
 #ifdef HAVE_TERMIOS_H
-static struct termios oldterm;
-		/* The user's original terminal settings. */
+static struct termios original_state;
+		/* The original settings of the user's terminal. */
 #else
 # define tcsetattr(...)
 # define tcgetattr(...)
@@ -100,7 +100,7 @@ linestruct *copy_node(const linestruct *src)
 {
 	linestruct *dst = nmalloc(sizeof(linestruct));
 
-	dst->data = mallocstrcpy(NULL, src->data);
+	dst->data = copy_of(src->data);
 	dst->next = src->next;
 	dst->prev = src->prev;
 	dst->lineno = src->lineno;
@@ -127,33 +127,33 @@ void splice_node(linestruct *afterthis, linestruct *newnode)
 }
 
 /* Disconnect a node from a linked list of linestructs and delete it. */
-void unlink_node(linestruct *fileptr)
+void unlink_node(linestruct *line)
 {
-	if (fileptr->prev != NULL)
-		fileptr->prev->next = fileptr->next;
-	if (fileptr->next != NULL)
-		fileptr->next->prev = fileptr->prev;
+	if (line->prev != NULL)
+		line->prev->next = line->next;
+	if (line->next != NULL)
+		line->next->prev = line->prev;
 
 	/* Update filebot when removing a node at the end of file. */
-	if (openfile && openfile->filebot == fileptr)
-		openfile->filebot = fileptr->prev;
+	if (openfile && openfile->filebot == line)
+		openfile->filebot = line->prev;
 
-	delete_node(fileptr);
+	delete_node(line);
 }
 
 /* Free the data structures in the given node. */
-void delete_node(linestruct *fileptr)
+void delete_node(linestruct *line)
 {
 #ifdef ENABLE_WRAPPING
 	/* If the spill-over line for hard-wrapping is deleted... */
-	if (fileptr == openfile->spillage_line)
+	if (line == openfile->spillage_line)
 		openfile->spillage_line = NULL;
 #endif
-	free(fileptr->data);
+	free(line->data);
 #ifdef ENABLE_COLOR
-	free(fileptr->multidata);
+	free(line->multidata);
 #endif
-	free(fileptr);
+	free(line);
 }
 
 /* Duplicate an entire linked list of linestructs. */
@@ -226,24 +226,24 @@ void partition_buffer(linestruct *top, size_t top_x,
 	 * top of the partition from it, and save the text before top_x. */
 	foreline = top->prev;
 	top->prev = NULL;
-	antedata = mallocstrncpy(NULL, top->data, top_x + 1);
+	antedata = measured_copy(top->data, top_x + 1);
 	antedata[top_x] = '\0';
 
 	/* Remember which line is below the bottom of the partition, detach the
 	 * bottom of the partition from it, and save the text after bot_x. */
 	hindline = bot->next;
 	bot->next = NULL;
-	postdata = mallocstrcpy(NULL, bot->data + bot_x);
+	postdata = copy_of(bot->data + bot_x);
 
 	/* At the end of the partition, remove all text after bot_x. */
 	bot->data[bot_x] = '\0';
 
 	/* At the beginning of the partition, remove all text before top_x. */
-	charmove(top->data, top->data + top_x, strlen(top->data) - top_x + 1);
+	memmove(top->data, top->data + top_x, strlen(top->data) - top_x + 1);
 }
 
 /* Unpartition the current buffer so that it is complete again. */
-void unpartition_buffer()
+void unpartition_buffer(void)
 {
 	/* Reattach the line that was above the top of the partition. */
 	openfile->filetop->prev = foreline;
@@ -253,9 +253,9 @@ void unpartition_buffer()
 	/* Restore the text that was on the first partition line before its start. */
 	openfile->filetop->data = charealloc(openfile->filetop->data,
 				strlen(antedata) + strlen(openfile->filetop->data) + 1);
-	charmove(openfile->filetop->data + strlen(antedata),
+	memmove(openfile->filetop->data + strlen(antedata),
 				openfile->filetop->data, strlen(openfile->filetop->data) + 1);
-	strncpy(openfile->filetop->data, antedata, strlen(antedata));
+	memcpy(openfile->filetop->data, antedata, strlen(antedata));
 	free(antedata);
 	antedata = NULL;
 
@@ -288,10 +288,8 @@ void extract(linestruct *top, size_t top_x, linestruct *bot, size_t bot_x)
 	bool mark_inside = FALSE;
 	bool same_line = FALSE;
 
-	if (top == bot && top_x == bot_x) {
-		statusline(ALERT, "Extracting nothing -- please report a bug");
+	if (top == bot && top_x == bot_x)
 		return;
-	}
 #endif
 
 	/* Partition the buffer so that it contains only the text from
@@ -344,7 +342,7 @@ void extract(linestruct *top, size_t top_x, linestruct *bot, size_t bot_x)
 
 	/* Since the text has now been saved, remove it from the file buffer. */
 	openfile->filetop = make_new_node(NULL);
-	openfile->filetop->data = mallocstrcpy(NULL, "");
+	openfile->filetop->data = copy_of("");
 	openfile->filebot = openfile->filetop;
 
 	/* Restore the current line and cursor position.  If the mark begins
@@ -465,11 +463,15 @@ void print_view_warning(void)
 	statusbar(_("Key is invalid in view mode"));
 }
 
-/* Indicate that something is disabled in restricted mode. */
-void show_restricted_warning(void)
+/* When in restricted mode, show a warning and return TRUE. */
+bool in_restricted_mode(void)
 {
-	statusbar(_("This function is disabled in restricted mode"));
-	beep();
+	if (ISSET(RESTRICTED)) {
+		statusbar(_("This function is disabled in restricted mode"));
+		beep();
+		return TRUE;
+	} else
+		return FALSE;
 }
 
 #ifndef ENABLE_HELP
@@ -500,9 +502,9 @@ void finish(void)
 	endwin();
 
 	/* Restore the old terminal settings. */
-	tcsetattr(0, TCSANOW, &oldterm);
+	tcsetattr(0, TCSANOW, &original_state);
 
-#ifdef ENABLE_NANORC
+#if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
 	display_rcfile_errors();
 #endif
 
@@ -531,7 +533,7 @@ void die(const char *msg, ...)
 	endwin();
 
 	/* Restore the old terminal settings. */
-	tcsetattr(0, TCSANOW, &oldterm);
+	tcsetattr(0, TCSANOW, &original_state);
 
 #ifdef ENABLE_NANORC
 	display_rcfile_errors();
@@ -651,11 +653,12 @@ void window_init(void)
 
 #ifdef ENABLED_WRAPORJUSTIFY
 	/* Set up the wrapping point, accounting for screen width when negative. */
-	wrap_at = fill;
-	if (wrap_at <= 0)
-		wrap_at += COLS;
-	if (wrap_at < 0)
+	if (COLS + fill < 0)
 		wrap_at = 0;
+	else if (fill <= 0)
+		wrap_at = COLS + fill;
+	else
+		wrap_at = fill;
 #endif
 }
 
@@ -685,7 +688,18 @@ void mouse_init(void)
 /* Print the usage line for the given option to the screen. */
 void print_opt(const char *shortflag, const char *longflag, const char *desc)
 {
-	printf(" %-14s %-23s %s\n", shortflag, longflag, _(desc));
+	int firstwidth = breadth(shortflag);
+	int secondwidth = breadth(longflag);
+
+	printf(" %s", shortflag);
+	if (firstwidth < 14)
+		printf("%*s", 14 - firstwidth, " ");
+
+	printf(" %s", longflag);
+	if (secondwidth < 24)
+		printf("%*s", 24 - secondwidth, " ");
+
+	printf("%s\n", _(desc));
 }
 
 /* Explain how to properly use nano and its command-line options. */
@@ -698,7 +712,7 @@ void usage(void)
 				"a '+' before the filename.  The column number can be added after a comma.\n"));
 	printf(_("When a filename is '-', nano reads data from standard input.\n\n"));
 	/* TRANSLATORS: The next three are column headers of the --help output. */
-	printf("%-16s%-26s%s\n", _("Option"), _("Long option"), _("Meaning"));
+	print_opt(_("Option"), _("Long option"), N_("Meaning"));
 #ifndef NANO_TINY
 	/* TRANSLATORS: The next forty or so strings are option descriptions
 	 * for the --help output.  Try to keep them at most 40 characters. */
@@ -730,7 +744,7 @@ void usage(void)
 	print_opt("-I", "--ignorercfiles", N_("Don't look at nanorc files"));
 #endif
 #ifndef NANO_TINY
-	print_opt("-J <number>", "--guidestripe=<number>",
+	print_opt(_("-J <number>"), _("--guidestripe=<number>"),
 					N_("Show a guiding bar at this column"));
 #endif
 	print_opt("-K", "--rawsequences",
@@ -754,7 +768,9 @@ void usage(void)
 #endif
 #ifdef ENABLE_JUSTIFY
 	print_opt(_("-Q <regex>"), _("--quotestr=<regex>"),
-					 N_("Regular expression to match quoting"));
+					/* TRANSLATORS: This refers to email quoting,
+					 * like the > in: > quoted text. */
+					N_("Regular expression to match quoting"));
 #endif
 	if (!ISSET(RESTRICTED))
 		print_opt("-R", "--restricted", N_("Restricted mode"));
@@ -1070,7 +1086,7 @@ void reconnect_and_store_state(void)
 
 	/* If input was not cut short, store the current state of the terminal. */
 	if (!control_C_was_pressed)
-		tcgetattr(0, &oldterm);
+		tcgetattr(0, &original_state);
 }
 
 /* Read whatever comes from standard input into a new buffer. */
@@ -1080,7 +1096,7 @@ bool scoop_stdin(void)
 
 	/* Exit from curses mode and put the terminal into its original state. */
 	endwin();
-	tcsetattr(0, TCSANOW, &oldterm);
+	tcsetattr(0, TCSANOW, &original_state);
 
 	/* When input comes from a terminal, show a helpful message. */
 	if (isatty(STANDARD_INPUT))
@@ -1205,7 +1221,7 @@ RETSIGTYPE do_suspend(int signal)
 	fflush(stdout);
 
 	/* Restore the old terminal settings. */
-	tcsetattr(0, TCSANOW, &oldterm);
+	tcsetattr(0, TCSANOW, &original_state);
 
 	/* The suspend keystroke must not elicit cursor-position display. */
 	suppress_cursorpos=TRUE;
@@ -1328,10 +1344,8 @@ void do_toggle(int flag)
 {
 	bool enabled;
 
-	if (flag == SUSPEND && ISSET(RESTRICTED)) {
-		show_restricted_warning();
+	if (flag == SUSPEND && in_restricted_mode())
 		return;
-	}
 
 	TOGGLE(flag);
 	focusing = FALSE;
@@ -1380,12 +1394,12 @@ void do_toggle(int flag)
 void disable_extended_io(void)
 {
 #ifdef HAVE_TERMIOS_H
-	struct termios term = {0};
+	struct termios settings = {0};
 
-	tcgetattr(0, &term);
-	term.c_lflag &= ~IEXTEN;
-	term.c_oflag &= ~OPOST;
-	tcsetattr(0, TCSANOW, &term);
+	tcgetattr(0, &settings);
+	settings.c_lflag &= ~IEXTEN;
+	settings.c_oflag &= ~OPOST;
+	tcsetattr(0, TCSANOW, &settings);
 #endif
 }
 
@@ -1393,11 +1407,11 @@ void disable_extended_io(void)
 void disable_kb_interrupt(void)
 {
 #ifdef HAVE_TERMIOS_H
-	struct termios term = {0};
+	struct termios settings = {0};
 
-	tcgetattr(0, &term);
-	term.c_lflag &= ~ISIG;
-	tcsetattr(0, TCSANOW, &term);
+	tcgetattr(0, &settings);
+	settings.c_lflag &= ~ISIG;
+	tcsetattr(0, TCSANOW, &settings);
 #endif
 }
 
@@ -1405,11 +1419,11 @@ void disable_kb_interrupt(void)
 void enable_kb_interrupt(void)
 {
 #ifdef HAVE_TERMIOS_H
-	struct termios term = {0};
+	struct termios settings = {0};
 
-	tcgetattr(0, &term);
-	term.c_lflag |= ISIG;
-	tcsetattr(0, TCSANOW, &term);
+	tcgetattr(0, &settings);
+	settings.c_lflag |= ISIG;
+	tcsetattr(0, TCSANOW, &settings);
 #endif
 }
 
@@ -1417,11 +1431,11 @@ void enable_kb_interrupt(void)
 void disable_flow_control(void)
 {
 #ifdef HAVE_TERMIOS_H
-	struct termios term;
+	struct termios settings;
 
-	tcgetattr(0, &term);
-	term.c_iflag &= ~IXON;
-	tcsetattr(0, TCSANOW, &term);
+	tcgetattr(0, &settings);
+	settings.c_iflag &= ~IXON;
+	tcsetattr(0, TCSANOW, &settings);
 #endif
 }
 
@@ -1429,11 +1443,11 @@ void disable_flow_control(void)
 void enable_flow_control(void)
 {
 #ifdef HAVE_TERMIOS_H
-	struct termios term;
+	struct termios settings;
 
-	tcgetattr(0, &term);
-	term.c_iflag |= IXON;
-	tcsetattr(0, TCSANOW, &term);
+	tcgetattr(0, &settings);
+	settings.c_iflag |= IXON;
+	tcsetattr(0, TCSANOW, &settings);
 #endif
 }
 
@@ -1454,27 +1468,27 @@ void terminal_init(void)
 	 * disable the special control keys and interpretation of the flow
 	 * control characters using termios, save the terminal state after
 	 * the first call, and restore it on subsequent calls. */
-	static struct termios newterm;
-	static bool newterm_set = FALSE;
+	static struct termios desired_state;
+	static bool have_new_state = FALSE;
 
-	if (!newterm_set) {
+	if (!have_new_state) {
 #endif
 		raw();
 		nonl();
 		noecho();
 		disable_extended_io();
+		disable_kb_interrupt();
+
 		if (ISSET(PRESERVE))
 			enable_flow_control();
-
-		disable_kb_interrupt();
 #ifdef USE_SLANG
-		if (!ISSET(PRESERVE))
+		else
 			disable_flow_control();
 
-		tcgetattr(0, &newterm);
-		newterm_set = TRUE;
+		tcgetattr(0, &desired_state);
+		have_new_state = TRUE;
 	} else
-		tcsetattr(0, TCSANOW, &newterm);
+		tcsetattr(0, TCSANOW, &desired_state);
 #endif
 }
 
@@ -1799,7 +1813,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 			output[i] = '\n';
 
 		/* Get the next multibyte character. */
-		charlen = parse_mbchar(output + i, onechar, NULL);
+		charlen = collect_char(output + i, onechar);
 
 		i += charlen;
 
@@ -1810,7 +1824,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 		/* Make room for the new character and copy it into the line. */
 		openfile->current->data = charealloc(openfile->current->data,
 										current_len + charlen + 1);
-		charmove(openfile->current->data + openfile->current_x + charlen,
+		memmove(openfile->current->data + openfile->current_x + charlen,
 						openfile->current->data + openfile->current_x,
 						current_len - openfile->current_x + 1);
 		strncpy(openfile->current->data + openfile->current_x, onechar,
@@ -1825,7 +1839,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 		if (openfile->last_action != ADD ||
 				openfile->current_undo->mark_begin_lineno != openfile->current->lineno ||
 				openfile->current_undo->mark_begin_x != openfile->current_x)
-			add_undo(ADD);
+			add_undo(ADD, NULL);
 
 		/* Note that current_x has not yet been incremented. */
 		if (openfile->current == openfile->mark &&
@@ -1993,7 +2007,7 @@ int main(int argc, char **argv)
 #endif
 
 	/* Back up the terminal settings so that they can be restored. */
-	tcgetattr(0, &oldterm);
+	tcgetattr(0, &original_state);
 
 	/* Get the state of standard input and ensure it uses blocking mode. */
 	stdin_flags = fcntl(0, F_GETFL, 0);
@@ -2317,18 +2331,8 @@ int main(int argc, char **argv)
 #ifdef ENABLE_SPELLER
 		alt_speller = NULL;
 #endif
-
-//#define TIMEIT 12
-#ifdef TIMEIT
-#include <time.h>
-	clock_t start = clock();
-#endif
 		/* Now process the system's and the user's nanorc file, if any. */
 		do_rcfiles();
-#ifdef TIMEIT
-		fprintf(stderr, "Took: %.3f\n",
-						 (double)(clock() - start) / CLOCKS_PER_SEC);
-#endif
 
 		/* If the backed-up command-line options have a value, restore them. */
 #ifdef ENABLED_WRAPORJUSTIFY
@@ -2440,11 +2444,11 @@ int main(int argc, char **argv)
 #ifdef ENABLE_JUSTIFY
 	/* Set the default value for things that weren't specified. */
 	if (punct == NULL)
-		punct = mallocstrcpy(NULL, "!.?");
+		punct = copy_of("!.?");
 	if (brackets == NULL)
-		brackets = mallocstrcpy(NULL, "\"')>]}");
+		brackets = copy_of("\"')>]}");
 	if (quotestr == NULL)
-		quotestr = mallocstrcpy(NULL, "^([ \t]*([!#%:;>|}]|/{2}))+");
+		quotestr = copy_of("^([ \t]*([!#%:;>|}]|/{2}))+");
 
 	/* Compile the quoting regex, and exit when it's invalid. */
 	quoterc = regcomp(&quotereg, quotestr, NANO_REG_EXTENDED);
@@ -2468,14 +2472,14 @@ int main(int argc, char **argv)
 		const char *spellenv = getenv("SPELL");
 
 		if (spellenv != NULL)
-			alt_speller = mallocstrcpy(NULL, spellenv);
+			alt_speller = copy_of(spellenv);
 	}
 #endif
 
 #ifndef NANO_TINY
 	/* If matchbrackets wasn't specified, set its default value. */
 	if (matchbrackets == NULL)
-		matchbrackets = mallocstrcpy(NULL, "(<[{)>]}");
+		matchbrackets = copy_of("(<[{)>]}");
 
 	/* If the whitespace option wasn't specified, set its default value. */
 	if (whitespace == NULL) {
@@ -2483,13 +2487,13 @@ int main(int argc, char **argv)
 		if (using_utf8()) {
 			/* A tab is shown as a Right-Pointing Double Angle Quotation Mark
 			 * (U+00BB), and a space as a Middle Dot (U+00B7). */
-			whitespace = mallocstrcpy(NULL, "\xC2\xBB\xC2\xB7");
+			whitespace = copy_of("\xC2\xBB\xC2\xB7");
 			whitelen[0] = 2;
 			whitelen[1] = 2;
 		} else
 #endif
 		{
-			whitespace = mallocstrcpy(NULL, ">.");
+			whitespace = copy_of(">.");
 			whitelen[0] = 1;
 			whitelen[1] = 1;
 		}
@@ -2497,7 +2501,7 @@ int main(int argc, char **argv)
 #endif /* !NANO_TINY */
 
 	/* Initialize the search string. */
-	last_search = mallocstrcpy(NULL, "");
+	last_search = copy_of("");
 	UNSET(BACKWARDS_SEARCH);
 
 	/* If tabsize wasn't specified, set its default value. */
@@ -2583,10 +2587,37 @@ int main(int argc, char **argv)
 	/* Read the files mentioned on the command line into new buffers. */
 	while (optind < argc && (!openfile || read_them_all)) {
 		ssize_t givenline = 0, givencol = 0;
-
+#ifndef NANO_TINY
+		char *searchstring = NULL;
+#endif
 		/* If there's a +LINE[,COLUMN] argument here, eat it up. */
 		if (optind < argc - 1 && argv[optind][0] == '+') {
-			if (!parse_line_column(&argv[optind++][1], &givenline, &givencol))
+			int n = 1;
+
+#ifndef NANO_TINY
+			while (isalpha(argv[optind][n])) {
+				switch (argv[optind][n++]) {
+					case 'c': SET(CASE_SENSITIVE); break;
+					case 'C': UNSET(CASE_SENSITIVE); break;
+					case 'r': SET(USE_REGEXP); break;
+					case 'R': UNSET(USE_REGEXP); break;
+					default:
+						statusline(ALERT, _("Invalid search modifier '%c'"),
+											argv[optind][n - 1]);
+				}
+			}
+
+			if (argv[optind][n] == '/' || argv[optind][n] == '?') {
+				if (argv[optind][n + 1]) {
+					searchstring = copy_of(&argv[optind][n + 1]);
+					if (argv[optind][n] == '?')
+						SET(BACKWARDS_SEARCH);
+				} else if (n == 1)
+					statusline(ALERT, _("Empty search string"));
+				optind++;
+			} else
+#endif
+			if (!parse_line_column(&argv[optind++][n], &givenline, &givencol))
 				statusline(ALERT, _("Invalid line or column number"));
 		}
 
@@ -2602,6 +2633,23 @@ int main(int argc, char **argv)
 		/* If a position was given on the command line, go there. */
 		if (givenline != 0 || givencol != 0)
 			do_gotolinecolumn(givenline, givencol, FALSE, FALSE);
+#ifndef NANO_TINY
+		else if (searchstring != NULL) {
+			if (ISSET(USE_REGEXP))
+				regexp_init(searchstring);
+			if (!findnextstr(searchstring, FALSE, JUSTFIND, NULL,
+							ISSET(BACKWARDS_SEARCH), openfile->filetop, 0))
+				not_found_msg(searchstring);
+			else if (lastmessage == HUSH)
+				wipe_statusbar();
+			openfile->placewewant = xplustabs();
+			if (ISSET(USE_REGEXP))
+				tidy_up_after_search();
+			free(last_search);
+			last_search = searchstring;
+			searchstring = NULL;
+		}
+#endif
 #ifdef ENABLE_HISTORIES
 		else if (ISSET(POSITIONLOG) && openfile->filename[0] != '\0') {
 			ssize_t savedline, savedcol;
@@ -2637,8 +2685,8 @@ int main(int argc, char **argv)
 	prepare_for_display();
 
 #ifdef ENABLE_NANORC
-	if (rcfile_with_errors != NULL)
-		statusline(ALERT, _("Mistakes in '%s'"), rcfile_with_errors);
+	if (startup_problem != NULL)
+		statusline(ALERT, startup_problem);
 #endif
 
 #ifdef ENABLE_HELP
